options(contrasts=c("contr.sum","contr.poly"))
options(warn=-1)

options(scipen=999)
options(width=256)

library("gdata")


library("ROCR")
library("pROC")
library("randomForest")
library("caret")

library("e1071")

library("pracma")
library("tcltk")

# clear all
rm(list=ls())

algorithm = 1
rets <- c("threshold", "sensitivity", "specificity", "ppv", "npv", "accuracy")
policy = "omit"
n_boot = 4000
thresholds = "local maxima"

min_importance = 1

d = read.csv("./database.csv",sep=",")
d$class = factor(d$class)

Niterations = 25 ### HOW MANY VARIABLES
Nsplit = dim(d)[1]	 ### HOW MANY train/test splits for each iteration
seed = 42

set.seed(seed)

n_show = 20

X = d[,c(3:(dim(d)[2]))]
writeLines("Using AUC feature importances...")
imp<-readRDS("importance_AUC.RDS")
bvars = imp$Variable
print(imp[c(1:n_show),])

best_acc = 0
best_mdl = {}
best_no_of_vars = 0
best_vars = {}
BEST_MODELS = {}

subj = c(1:dim(X)[1])

writeLines("Training incremental models...")
j = 1
while(j<=Niterations){
	d_sub <- d[,c("class",bvars[c(1:j)])]	
	PREDS = zeros(dim(X)[1],1)
	PROBS = zeros(dim(X)[1],1)
	for (k in subj){
		x_new = as.data.frame(d_sub[-k,c(2:(j+1))])
		colnames(x_new) = bvars[c(1:j)]
		y_new = d_sub$class[-k]
		x_test = as.data.frame(d_sub[k,c(2:(j+1))])
		colnames(x_test) = bvars[c(1:j)]
		mdl <- glm(class~., data=d_sub[-k,], family = binomial(link=logit))
		p_roc = roc(cases=as.numeric(mdl$fitted.values)[y_new=="RBD"],controls=as.numeric(mdl$fitted.values)[y_new=="HC"],quiet=TRUE)
		res = coords(p_roc, x="best", input="threshold", ret=rets, best.policy = policy)
		pred = predict(mdl, x_test, type="response")
		PROBS[k] = pred[[1]]
		if(pred[[1]]>=res$threshold){
			PREDS[k] = 1
		}else{
			PREDS[k] = 0
		}
	}
	mdl <- glm(class~., data=d_sub, family = binomial(link=logit))
	x = as.data.frame(d_sub[,bvars[c(1:j)]])
	colnames(x) <- bvars[c(1:j)]
	predictions = predict(mdl, x, type="response")
	PREDS = as.factor(PREDS)
	y_ref = as.factor(as.numeric(d$class)-1)
	cc = confusionMatrix(PREDS,y_ref,positive="1")
	sens = cc$byClass[1]
	spec = cc$byClass[2]
	acc  = cc$overall[1]
	kappa = cc$overall[2]
	acc_low = cc$overall[3]
	acc_up = cc$overall[4]
	roc_c = roc(as.numeric(y_ref)-1,PROBS,quiet=TRUE)
	auc = roc_c$auc
	fprintf("No. of variables: %d, accuracy=%.2f(%.2f,%.2f), sens=%.2f, spec=%.2f, AUC=%.2f\n",j,acc,acc_low,acc_up,sens,spec,auc)
	if(acc>best_acc){
		best_acc = acc
		best_no_of_vars = j
		best_vars = bvars[c(1:j)]
		#pred = predict(best_model, newdata=d[,c("class",best_vars)], type="response")
		#cc = confusionMatrix(pred,d$class,positive='RBD')
		fname = gsub(" ","",paste('model_glm_',j,'vars.rdata'))
		#fprintf("*** Accuracy on whole training dataset = %.2f\n",cc$overall[1])
		save(PREDS, PROBS, predictions, mdl, y_ref, roc_c, best_vars, acc, file=fname)		
	}
	j = j+1
}

