options(contrasts=c("contr.sum","contr.poly"))
options(warn=-1)

options(scipen=999)
options(width=256)

library("gdata")


library("ROCR")
library("pROC")
library("randomForest")
library("caret")

library("e1071")

library("pracma")

# clear all
rm(list=ls())

min_importance = 1

d = read.csv("./database.csv",sep=",")

d$class = factor(d$class)

Niterations = 20 ### HOW MANY VARIABLES
Nsplit = dim(d)[1]	 ### HOW MANY train/test splits for each iteration
seed = 42

set.seed(seed)

n_show = 20

fitControl <- trainControl(
  method = 'LOOCV',                
  number = 1,                     
  #savePredictions = 'final',        
  classProbs = TRUE,
  seed = as.list(rep(1,1+dim(d)[1])),                
  summaryFunction = twoClassSummary
)

tunegrid <- expand.grid(.mtry=floor(sqrt(ncol(d)-1)))

# Random Forest: choose features with the highest importance
writeLines("Evaluating feature importances...")

X = d[,c(3:(dim(d)[2]))]
rf <- train(class~., data=d[,c(2:dim(d)[2])], method='rf', type=1, preProc=c("center", "scale"), tuneGrid=tunegrid, trControl = fitControl)

imp <- varImp(rf,scale=FALSE)
#saveRDS(imp,file="importance_RF_ALL.rds")
i_imp = order(imp$importance,decreasing=TRUE)
bvars = colnames(X[i_imp])
print(cbind(bvars[c(1:n_show)],round(imp$importance[i_imp[c(1:n_show)],],2)))

best_acc = 0
best_auc = 0
best_mdl = {}
best_no_of_vars = 0
best_vars = {}
BEST_MODELS = {}

fitControl <- trainControl(
  method = 'LOOCV',                
  number = 1,                     
  savePredictions = TRUE,        
  classProbs = T,
  seed = as.list(rep(1,1+dim(d)[1])),                
  summaryFunction=twoClassSummary 
)

writeLines("Training models...")
j = 1
while(j<=Niterations){
	d_sub <- d[,c("class",bvars[c(1:j)])]
	tunegrid <- expand.grid(.mtry=floor(sqrt(ncol(d_sub)-1)))
	out <- train(class~., data=d_sub, method='rf', preProc=c("center", "scale"), tuneGrid=tunegrid, trControl = fitControl)
	cc = confusionMatrix(out$finalModel$predicted,out$finalModel$y,positive="RBD")
	sens = cc$byClass[1]
	spec = cc$byClass[2]
	acc  = cc$overall[1]
	kappa = cc$overall[2]
	acc_low = cc$overall[3]
	acc_up = cc$overall[4]
	roc_c = roc(out$finalModel$y,out$finalModel$votes[,2],quiet=TRUE)
	auc = roc_c$auc
	fprintf("No. of variables: %d, accuracy=%.2f(%.2f,%.2f), AUC=%.2f, sens=%.2f, spec=%.2f\n",j,acc,acc_low,acc_up,auc,sens,spec)
	if(acc>best_acc){
		best_acc = acc
		best_auc = auc
		best_no_of_vars = j
		best_vars = bvars[c(1:j)]
		fname = gsub(" ","",paste('model_rf_mean_',j,'vars.rdata'))
		save(out, best_vars, acc, file=fname)		
	}
	j = j+1
}

