## generate plots
library("pROC")
library("ggplot2")
library("graphics")
library("gridExtra")
library("ggpubr")
source("plotVarImp.R")


imp_rf = readRDS("importance_RF_ALL.rds")
imp_xgb = readRDS("importance_XGB_ALL.rds")
imp_roc = readRDS("importance_ROC.rds")

imp1 <- data.frame("Variable"=rownames(imp_rf$importance),"importance"=imp_rf$importance$Overall)
imp2 <- data.frame("Variable"=rownames(imp_xgb$importance),"importance"=imp_xgb$importance$Overall)
imp3 <- data.frame("Variable"=rownames(imp_roc$importance),"importance"=imp_roc$importance$RBD)
p1 <- plotVarImp(imp1,color="blue",top=15,scale=F)
p2 <- plotVarImp(imp2,color="blue",top=15,scale=F)
p3 <- plotVarImp(imp3,color="blue",top=15,scale=F)


### ROC curves, object g_roc is defined
source("plot_ROCs.r")
source("plot_calibration.r")

p_all <- ggarrange(p3+ggtitle("ROC AUC")+ylab("importance (AUC)"), p1+ggtitle("Random Forest"), p2+ggtitle("XGBoost"), 
          labels = c("a", "b", "c"),
          font.label = list(face="bold",size=16),
          ncol = 2, nrow = 2)
tiff("importances.tif",width=10,height=10,units="in",compression="lzw",res=300)
	print(p_all)
dev.off()

tiff("roc_curves.tif",width=6,height=5,units="in",compression="lzw",res=300)
	print(g_roc)		
dev.off()


p_cal_roc <- ggarrange(g_roc, g_cal, 
          labels = c("a", "b"),
          font.label = list(face="bold",size=16),
          ncol = 2, nrow = 1)
tiff("roc+cal_curves.tif",width=12,height=5,units="in",compression="lzw",res=300)
	print(p_cal_roc)
dev.off()

tiff("Figure1_boxplots.tif",width=12,height=6,units="in",compression="lzw",res=300)
	par(mfrow=c(1,2))
	boxplot(d$LF_PWR_mean_sw~d$class, xlab="", ylab="LF(night)/LF(day) ratio")
	mtext(expression(paste(bold("A."),"   Cardiac Sympathetic Index")),side=3,adj=0, line=2, cex=1.5)
	#box("outer",lty="solid",col="blue")
	boxplot(d$HF_PWR_mean_sw~d$class, xlab="", ylab="HF(night)/HF(day) ratio")
	mtext(expression(paste(bold("B."),"   Cardiac Parasympathetic Index")),side=3,adj=0, line=2, cex=1.5)
	#box("outer",lty="solid",col="blue",lwd=5)
dev.off()


library("pROC")
library("pracma")
algorithm = 1
rets <- c("threshold", "sensitivity", "specificity", "ppv", "npv", "accuracy")
policy = "random"
n_boot = 10000
thresholds = "local maxima"

writeLines("\nLF index (sympathetic)")
p_roc <- roc(cases=d$LF_PWR_mean_sw[d$class=="RBD"],controls=d$LF_PWR_mean_sw[d$class=="HC"],percent=FALSE,auc=TRUE,ci=TRUE,na.rm=TRUE,algorithm=algorithm,quiet=TRUE)
#auc = p_roc$auc[1]
#fprintf("AUC = %.2f\n",auc)
print(p_roc)
roc_res = ci.coords(p_roc, x="best", input="threshold", ret=rets, best.policy = policy, progress = "none",boot.n=n_boot)
print(roc_res)

writeLines("\n\nHF index (parasympathetic)")
p_roc <- roc(cases=d$HF_PWR_mean_sw[d$class=="RBD"],controls=d$HF_PWR_mean_sw[d$class=="HC"],percent=FALSE,auc=TRUE,ci=TRUE,na.rm=TRUE,algorithm=algorithm,quiet=TRUE)
#auc = p_roc$auc[1]	
#fprintf("AUC = %.2f\n",auc)
print(p_roc)
roc_res = ci.coords(p_roc, x="best", input="threshold", ret=rets, best.policy = policy, progress = "none",boot.n=n_boot)
print(roc_res)
